From cf6e4c7b52ff38c37f9a5054ce0387c0ef1b0d3c Mon Sep 17 00:00:00 2001
From: "Benjamin M. Schwartz" <bens@alum.mit.edu>
Date: Tue, 25 Dec 2012 22:56:08 -0800
Subject: [PATCH] Fixes for large file (>2 GB) support on MSVC and mingw.

These changes include:
 - 64-bit file access wrappers modeled on the corresponding
   functions in libopus.
 - Using oggz_off_t instead of long for offsets
 - Changing the definition of oggz_off_t to off64_t under MSVC.
 - Size-appropriate printf codes.

The changes are known to compile and pass tests (including merging
large files) under mingw32, linux-gcc, and MSVC 6.
---
 configure.ac                           | 10 +++++-----
 include/oggz/oggz_off_t.h              | 10 ++++++++--
 include/oggz/oggz_off_t_generated.h.in |  2 +-
 src/liboggz/oggz_io.c                  | 30 +++++++++++++++++++++++-----
 src/liboggz/oggz_private.h             | 10 +++++-----
 src/liboggz/oggz_seek.c                |  8 ++++----
 src/tools/oggz-info.c                  | 36 ++++++++++++++++++----------------
 src/tools/oggz_tools.c                 | 29 ++++++++++++---------------
 src/tools/oggz_tools.h                 |  4 ++--
 9 files changed, 82 insertions(+), 57 deletions(-)

diff --git a/configure.ac b/configure.ac
index 4d4694d..a3edd16 100644
--- a/configure.ac
+++ b/configure.ac
@@ -261,7 +261,11 @@ AC_CHECK_SIZEOF(off_t, 1)
 
 case "$host_os" in
     mingw*)
-        TYPEOF_OGGZ_OFF_T="__int64"
+        CFLAGS="$CFLAGS -D_FILE_OFFSET_BITS=64"
+        dnl CFLAGS="$CFLAGS -D_LARGEFILE64_SOURCE -D_FILE_OFFSET_BITS=64"
+        dnl AC_DEFINE([_LARGEFILE64_SOURCE], [], [])
+        dnl AC_DEFINE([_FILE_OFFSET_BITS], [64], [])
+        TYPEOF_OGGZ_OFF_T="int64_t"
         OGGZ_OFF_MAX="0x7FFFFFFFFFFFFFFFLL"
         SIZEOF_OGGZ_OFF_T=8
         ;;
@@ -326,9 +330,6 @@ case "$host_os" in
 
 if test $SIZEOF_OGGZ_OFF_T = 4 ; then
     OGGZ_OFF_MAX="0x7FFFFFFF"
-    PRI_OGGZ_OFF_T="l"
-elif test $SIZEOF_OGGZ_OFF_T = 8 ; then
-    PRI_OGGZ_OFF_T="ll"
 fi
 
 dnl The following configured variables are written into the public header
@@ -337,7 +338,6 @@ dnl   oggz_off_t (typedef)
 dnl   PRI_OGGZ_OFF_T (printf format)
 
 AC_SUBST(TYPEOF_OGGZ_OFF_T)
-AC_SUBST(PRI_OGGZ_OFF_T)
 
 dnl SIZEOF_OGGZ_OFF_T and OGGZ_OFF_MAX are only used internally by liboggz
 AC_DEFINE_UNQUOTED([SIZEOF_OGGZ_OFF_T],${SIZEOF_OGGZ_OFF_T}, [Set to sizeof (long) if unknown.])
diff --git a/include/oggz/oggz_off_t.h b/include/oggz/oggz_off_t.h
index 0a89f83..972b256 100644
--- a/include/oggz/oggz_off_t.h
+++ b/include/oggz/oggz_off_t.h
@@ -40,12 +40,18 @@
 
 #ifdef _WIN32
   /* MSVC/Borland & Cygwin */
-   typedef off_t oggz_off_t;
+  typedef __int64 oggz_off_t; 
 
-#define PRI_OGGZ_OFF_T "l"
+#define PRI_OGGZ_INT64_T "I64"
 
 #else
 #include <oggz/oggz_off_t_generated.h>
 #endif
 
+#if SIZEOF_OGGZ_OFF_T == 8
+#define PRI_OGGZ_OFF_T PRI_OGGZ_INT64_T
+#else
+#define PRI_OGGZ_OFF_T ""
+#endif
+
 #endif /* __OGGZ_OFF_T__ */
diff --git a/include/oggz/oggz_off_t_generated.h.in b/include/oggz/oggz_off_t_generated.h.in
index 0163773..c085021 100644
--- a/include/oggz/oggz_off_t_generated.h.in
+++ b/include/oggz/oggz_off_t_generated.h.in
@@ -61,6 +61,6 @@
 
 typedef @TYPEOF_OGGZ_OFF_T@ oggz_off_t;
 
-#define PRI_OGGZ_OFF_T "@PRI_OGGZ_OFF_T@"
+#define PRI_OGGZ_INT64_T "ll"
 
 #endif /* __OGGZ_OFF_T_GENERATED__ */
diff --git a/src/liboggz/oggz_io.c b/src/liboggz/oggz_io.c
index 9a139a9..bac6b1e 100644
--- a/src/liboggz/oggz_io.c
+++ b/src/liboggz/oggz_io.c
@@ -94,13 +94,23 @@ oggz_io_write (OGGZ * oggz, void * buf, size_t n)
   return bytes;
 }
 
+static int
+oggz_fseek (FILE * _stream, oggz_off_t _offset, int _whence)
+{
+#if defined (_MSC_VER)
+  return _fseeki64 (_stream, _offset, _whence);
+#else
+  return fseeko (_stream, _offset, _whence);
+#endif
+}
+
 int
-oggz_io_seek (OGGZ * oggz, long offset, int whence)
+oggz_io_seek (OGGZ * oggz, oggz_off_t offset, int whence)
 {
   OggzIO * io;
 
   if (oggz->file != NULL) {
-    if (fseek (oggz->file, offset, whence) == -1) {
+    if (oggz_fseek (oggz->file, offset, whence) == -1) {
       if (errno == ESPIPE) {
 	/*oggz_set_error (oggz, OGGZ_ERR_NOSEEK);*/
       } else {
@@ -121,14 +131,24 @@ oggz_io_seek (OGGZ * oggz, long offset, int whence)
   return 0;
 }
 
-long
+static oggz_off_t
+oggz_ftell (FILE *_stream)
+{
+#if defined (_MSC_VER)
+  return _ftelli64 (_stream);
+#else
+  return ftello64 (_stream);
+#endif
+}
+
+oggz_off_t
 oggz_io_tell (OGGZ * oggz)
 {
   OggzIO * io;
-  long offset;
+  oggz_off_t offset;
 
   if (oggz->file != NULL) {
-    if ((offset = ftell (oggz->file)) == -1) {
+    if ((offset = oggz_ftell (oggz->file)) == -1) {
       if (errno == ESPIPE) {
 	/*oggz_set_error (oggz, OGGZ_ERR_NOSEEK);*/
       } else {
diff --git a/src/liboggz/oggz_private.h b/src/liboggz/oggz_private.h
index 7053571..cb2de12 100644
--- a/src/liboggz/oggz_private.h
+++ b/src/liboggz/oggz_private.h
@@ -46,7 +46,7 @@
 #include "oggz_vector.h"
 #include "oggz_dlist.h"
 
-#define OGGZ_AUTO_MULT 1000Ull
+#define OGGZ_AUTO_MULT ((ogg_int64_t)1000)
 
 typedef struct _OGGZ OGGZ;
 typedef struct _OggzComment OggzComment;
@@ -75,8 +75,8 @@ typedef int (*OggzWriteHungry) (OGGZ * oggz, int empty, void * user_data);
 /* oggz_io */
 typedef size_t (*OggzIORead) (void * user_handle, void * buf, size_t n);
 typedef size_t (*OggzIOWrite) (void * user_handle, void * buf, size_t n);
-typedef int (*OggzIOSeek) (void * user_handle, long offset, int whence);
-typedef long (*OggzIOTell) (void * user_handle);
+typedef int (*OggzIOSeek) (void * user_handle, oggz_off_t offset, int whence);
+typedef oggz_off_t (*OggzIOTell) (void * user_handle);
 typedef int (*OggzIOFlush) (void * user_handle);
 
 struct _oggz_stream_t {
@@ -324,8 +324,8 @@ long oggz_comments_encode (OGGZ * oggz, long serialno,
 /* oggz_io */
 size_t oggz_io_read (OGGZ * oggz, void * buf, size_t n);
 size_t oggz_io_write (OGGZ * oggz, void * buf, size_t n);
-int oggz_io_seek (OGGZ * oggz, long offset, int whence);
-long oggz_io_tell (OGGZ * oggz);
+int oggz_io_seek (OGGZ * oggz, oggz_off_t offset, int whence);
+oggz_off_t oggz_io_tell (OGGZ * oggz);
 int oggz_io_flush (OGGZ * oggz);
 
 /* oggz_read */
diff --git a/src/liboggz/oggz_seek.c b/src/liboggz/oggz_seek.c
index ba31e73..99c9652 100644
--- a/src/liboggz/oggz_seek.c
+++ b/src/liboggz/oggz_seek.c
@@ -141,7 +141,7 @@ oggz_reset_streams (OGGZ * oggz)
   oggz_vector_foreach (oggz->streams, oggz_stream_reset);
 }
 
-static long
+static oggz_off_t
 oggz_reset_seek (OGGZ * oggz, oggz_off_t offset, ogg_int64_t unit, int whence)
 {
   OggzReader * reader = &oggz->x.reader;
@@ -162,7 +162,7 @@ oggz_reset_seek (OGGZ * oggz, oggz_off_t offset, ogg_int64_t unit, int whence)
   return offset_at;
 }
 
-static long
+static oggz_off_t
 oggz_reset (OGGZ * oggz, oggz_off_t offset, ogg_int64_t unit, int whence)
 {
   oggz_reset_streams (oggz);
@@ -650,7 +650,7 @@ oggz_bounded_seek_set (OGGZ * oggz,
 #ifdef DEBUG
     printf ("oggz_bounded_seek_set: unit_target == reader->current_unit, SKIP\n");
 #endif
-    return (long)reader->current_unit;
+    return reader->current_unit;
   }
 
   if (unit_target == 0) {
@@ -776,7 +776,7 @@ oggz_bounded_seek_set (OGGZ * oggz,
   printf ("oggz_bounded_seek_set: FOUND (%lld)\n", unit_at);
 #endif
 
-  return (long)reader->current_unit;
+  return reader->current_unit;
 }
 
 static ogg_int64_t
diff --git a/src/tools/oggz-info.c b/src/tools/oggz-info.c
index 24dbc57..04441a0 100644
--- a/src/tools/oggz-info.c
+++ b/src/tools/oggz-info.c
@@ -108,20 +108,20 @@ struct _OI_Info {
   OGGZ * oggz;
   OggzTable * tracks;
   ogg_int64_t duration;
-  long length_total;
-  long overhead_length_total;
+  oggz_off_t length_total;
+  oggz_off_t overhead_length_total;
 };
 
 struct _OI_Stats {
   /* Pass 1 */
-  long count;
-  long length_total;
-  long length_min;
-  long length_max;
-  long overhead_length_total;
+  ogg_int64_t count;
+  oggz_off_t length_total;
+  oggz_off_t length_min;
+  oggz_off_t length_max;
+  oggz_off_t overhead_length_total;
 
   /* Pass 2 */
-  long length_avg;
+  oggz_off_t length_avg;
   ogg_int64_t length_deviation_total;
   double length_stddev;
 };
@@ -206,6 +206,7 @@ oi_stats_clear (OI_Stats * stats)
   stats->count = 0;
 
   stats->length_total = 0;
+  /* Strictly speaking this should be "OGGZ_OFF_T_MAX" */
   stats->length_min = LONG_MAX;
   stats->length_max = 0;
   stats->overhead_length_total = 0;
@@ -235,11 +236,11 @@ oggz_info_trackinfo_new (void)
   return oit;
 }
 
-static long
-oi_bitrate (long bytes, ogg_int64_t ms)
+static double
+oi_bitrate (oggz_off_t bytes, ogg_int64_t ms)
 {
   if (ms == 0) return 0;
-  else return (long) (((ogg_int64_t)bytes * 8 * 1000) / ms);
+  else return ((double)bytes * 8 * 1000) / ms;
 }
 
 static void
@@ -254,11 +255,11 @@ oi_stats_print (OI_Info * info, OI_Stats * stats, char * label)
   putchar ('\n');
 
 #if 0
-  printf ("\t%s-Length-Maximum: %ld bytes\n", label, stats->length_max);
-  /*printf ("\t%s-Length-Average: %ld bytes\n", label, stats->length_avg);*/
+  printf ("\t%s-Length-Maximum: %" PRI_OGGZ_OFF_T " bytes\n", label, stats->length_max);
+  /*printf ("\t%s-Length-Average: %" PRI_OGGZ_OFF_T " bytes\n", label, stats->length_avg);*/
   printf ("\t%s-Length-StdDev: %.0f bytes\n", label, stats->length_stddev);
   /*
-  printf ("\tRange: [%ld - %ld] bytes, Std.Dev. %.3f bytes\n",
+  printf ("\tRange: [%" PRI_OGGZ_OFF_T " - %" PRI_OGGZ_OFF_T "] bytes, Std.Dev. %.3f bytes\n",
 	  stats->length_min, stats->length_max, stats->length_stddev);
   */
 #endif
@@ -415,7 +416,7 @@ read_page_pass1 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_da
 {
   OI_Info * info = (OI_Info *)user_data;
   OI_TrackInfo * oit;
-  long bytes;
+  oggz_off_t bytes;
 
   oit = oggz_table_lookup (info->tracks, serialno);
   if (oit == NULL) {
@@ -451,7 +452,8 @@ read_page_pass2 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_da
 {
   OI_Info * info = (OI_Info *)user_data;
   OI_TrackInfo * oit;
-  long bytes, deviation;
+  oggz_off_t bytes;
+  ogg_int64_t deviation;
 
   oit = oggz_table_lookup (info->tracks, serialno);
 
@@ -512,7 +514,7 @@ read_packet_pass2 (OGGZ * oggz, oggz_packet * zp, long serialno,
   OI_Info * info = (OI_Info *)user_data;
   ogg_packet * op = &zp->op;
   OI_TrackInfo * oit;
-  long deviation;
+  ogg_int64_t deviation;
   
   oit = oggz_table_lookup (info->tracks, serialno);
 
diff --git a/src/tools/oggz_tools.c b/src/tools/oggz_tools.c
index 41d790f..21aca05 100644
--- a/src/tools/oggz_tools.c
+++ b/src/tools/oggz_tools.c
@@ -433,21 +433,21 @@ ot_page_identify (OGGZ *oggz, const ogg_page * og, char ** info)
  * using standard abbreviations (GB, MB, kB, byte[s])
  */
 int
-ot_fprint_bytes (FILE * stream, long nr_bytes)
+ot_fprint_bytes (FILE * stream, double nr_bytes)
 {
   if (nr_bytes > (1L<<30)) {
     return fprintf (stream, "%0.3f GB",
-		   (double)nr_bytes / (1024.0 * 1024.0 * 1024.0));
+		   nr_bytes / (1024.0 * 1024.0 * 1024.0));
   } else if (nr_bytes > (1L<<20)) {
     return fprintf (stream, "%0.3f MB",
-		   (double)nr_bytes / (1024.0 * 1024.0));
+		   nr_bytes / (1024.0 * 1024.0));
   } else if (nr_bytes > (1L<<10)) {
     return fprintf (stream, "%0.3f kB",
-		   (double)nr_bytes / (1024.0));
+		   nr_bytes / (1024.0));
   } else if (nr_bytes == 1) {
     return fprintf (stream, "1 byte");
   } else {
-    return fprintf (stream, "%ld bytes", nr_bytes);
+    return fprintf (stream, "%.0f bytes", nr_bytes);
   }
 }
 
@@ -456,19 +456,16 @@ ot_fprint_bytes (FILE * stream, long nr_bytes)
  * using quasi-standard abbreviations (Gbps, Mbps, kbps, bps)
  */
 int
-ot_print_bitrate (long bps)
+ot_print_bitrate (double bps)
 {
-  if (bps > (1000000000L)) {
-    return printf ("%0.3f Gbps",
-		   (double)bps / (1000.0 * 1000.0 * 1000.0));
-  } else if (bps > (1000000L)) {
-    return printf ("%0.3f Mbps",
-		   (double)bps / (1000.0 * 1000.0));
-  } else if (bps > (1000L)) {
-    return printf ("%0.3f kbps",
-		   (double)bps / (1000.0));
+  if (bps > 1.0e9) {
+    return printf ("%0.3f Gbps", bps / 1.0e9);
+  } else if (bps > 1.0e6) {
+    return printf ("%0.3f Mbps", bps / 1.0e6);
+  } else if (bps > 1.0e3) {
+    return printf ("%0.3f kbps", bps / 1.0e3);
   } else {
-    return printf ("%ld bps", bps);
+    return printf ("%0.0f bps", bps);
   }
 }
 
diff --git a/src/tools/oggz_tools.h b/src/tools/oggz_tools.h
index 53f670e..c1ab1f8 100644
--- a/src/tools/oggz_tools.h
+++ b/src/tools/oggz_tools.h
@@ -43,13 +43,13 @@ ot_page_identify (OGGZ *oggz, const ogg_page * og, char ** info);
  * Print a number of bytes to 3 significant figures
  * using standard abbreviations (GB, MB, kB, byte[s])
  */
-int ot_fprint_bytes (FILE * stream, long nr_bytes);
+int ot_fprint_bytes (FILE * stream, double nr_bytes);
 
 /*
  * Print a bitrate to 3 significant figures
  * using quasi-standard abbreviations (Gbps, Mbps, kbps, bps)
  */
-int ot_print_bitrate (long bps);
+int ot_print_bitrate (double bps);
 
 int ot_fprint_time (FILE * stream, double seconds);
 
-- 
1.8.0.1
